#' Make monthly temperature scenario from historic records
#' 
#' Produces a scenario containing monthly means for Tmin and Tmax that are representative
#' of a particular year. This scenario is computed by applying linear regression to a
#' file containing Tmin and Tmax records, and using the regression model to calculate typical
#' values for the user-specified year.
#' 
#' This function produces outputs that can be used as input for the temperature_generation
#' function. Sample applications are the use of the temperature_generation function for
#' making replicate weather records for a given year for risk assessment purposes, or
#' the generation of a weather scenario that can be compared with other datasets (e.g. climate
#' scenarios based on the WorldClim dataset refer to a 1951-2000 baseline, so that meaningful
#' use of such scenarios for local contexts requires consideration of a scenario that corresponds
#' to temperatures in 1975, the central year of this period).
#' 
#' 
#' @param weather daily weather, as produced with the fix_weather
#' function. Can also be generated by other means, but shouold contain the columns
#' c("Month","Day","Year","Tmin","Tmax").
#' @param year year, for which the scenario is to be produced
#' @param weather_start start year of the period to be considered in calculating the regression
#' @param weather_end end year of the period to be considered in calculating the regression
#' @return climate scenario object, consisting of the following elements: 'data' = a data frame with
#' n_intervals elements containing the absolute temperature information. 'reference_year' =
#' the year the scenario is representative of, i.e. the specified 'year' parameter. 'scenario_type'
#' = 'absolute' (because this is an absolute temperature scenario, not a relative one);
#' 'labels' = 'and elements attached to the input climate_scenario as an element names 'labels' =
#' 'regression-based scenario'.
#' 
#' @author Eike Luedeling
#' @keywords utility
#' @importFrom stats lm
#' @examples
#' 
#' temperature_scenario_from_records(weather=KA_weather,year=2001,weather_start=2000,weather_end=2005)
#' 
#'  
#' @export temperature_scenario_from_records
temperature_scenario_from_records<-function(weather,year,
                                            weather_start,weather_end)
{
  if(!is.data.frame(weather)) return(warning("Error - weather object is not a data.frame"))
  if(!"Tmin" %in% colnames(weather)|
     !"Tmax" %in% colnames(weather)|
     !"Day" %in% colnames(weather)|
     !"Month" %in% colnames(weather)|
     !"Year" %in% colnames(weather))
    return(warning("Error - required columns missing ('Day','Month','Year','Tmin' or 'Tmax')"))
  if(weather_start==weather_end) return(warning("Error - only one data year selected"))
  if(weather_start>weather_end) return(warning("Error - End year before start year"))
  
  if(!is.na(weather_start)) weather<-weather[which(weather$Year>=weather_start),]
  if(!is.na(weather_end)) weather<-weather[which(weather$Year<=weather_end),]
  
  if(nrow(weather)==0) return(warning("Error - no weather records in specified interval"))
  
  past_means<-list() 
  past_means[["Tmin"]]<-aggregate(weather$Tmin,by=list(weather$Year,weather$Month),FUN="mean")
  past_means[["Tmax"]]<-aggregate(weather$Tmax,by=list(weather$Year,weather$Month),FUN="mean")
  colnames(past_means[["Tmin"]])<-c("Year","Month","Temp")
  colnames(past_means[["Tmax"]])<-c("Year","Month","Temp")
  
  baseclim_baseline_duration_adjustment<-data.frame(Tmin=rep(NA,12),Tmax=rep(NA,12))
  rownames(baseclim_baseline_duration_adjustment)<-1:12
  for (v in c("Tmin","Tmax"))
    for (i in 1:12)
    {monthly<-past_means[[v]][which(past_means[[v]]$Month==i),]
    model<-lm(monthly$Temp~monthly$Year)
    baseclim_baseline_duration_adjustment[i,v]<-as.numeric(model$coefficients[1]+model$coefficients[2]*year)
    if(is.na(baseclim_baseline_duration_adjustment[i,v]))
      return(warning(paste("Error - unable to calculate regression for",v,"in month",i)))}
  return(list(data=baseclim_baseline_duration_adjustment,
              scenario_year=year,
              reference_year=NA,
              scenario_type="absolute",
              labels="regression-based scenario"))
}
